==> mule-runtime-tooling-client-4.0.0-sources.jar: org/mule/tooling/client/internal/ToolingMetadataServiceAdapter.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.tooling.client.internal;

import static com.google.common.base.Preconditions.checkNotNull;
import static java.lang.String.format;
import static java.util.Collections.emptyList;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.Optional.ofNullable;
import static org.mule.runtime.api.metadata.resolving.FailureCode.COMPONENT_NOT_FOUND;
import static org.mule.runtime.api.metadata.resolving.MetadataFailure.Builder.newFailure;
import static org.mule.runtime.api.metadata.resolving.MetadataResult.failure;
import static org.mule.runtime.api.util.Preconditions.checkState;
import static org.mule.tooling.client.internal.MetadataPartsFactory.toMetadataFailuresDTO;
import static org.mule.tooling.client.internal.MetadataPartsFactory.toMetadataKeysContainerDTO;
import static org.mule.tooling.client.internal.MetadataPartsFactory.toMetadataResultDTO;
import static org.mule.tooling.client.internal.serialization.SerializationUtilsImpl.deserialize;
import static org.mule.tooling.client.internal.serialization.SerializationUtilsImpl.serialize;
import org.mule.datasense.enrichment.model.IdComponentModelSelector;
import org.mule.runtime.api.metadata.MetadataKeysContainer;
import org.mule.runtime.api.metadata.resolving.MetadataResult;
import org.mule.runtime.api.util.Reference;
import org.mule.runtime.config.internal.model.ApplicationModel;
import org.mule.runtime.config.internal.model.ComponentModel;
import org.mule.tooling.client.api.component.location.Location;
import org.mule.tooling.client.api.datasense.ImmutableMetadataCacheKeyInfo;
import org.mule.tooling.client.api.datasense.ImmutableMetadataResult;
import org.mule.tooling.client.api.datasense.MetadataCache;
import org.mule.tooling.client.api.exception.ServiceUnavailableException;
import org.mule.tooling.client.api.metadata.MetadataFailure;
import org.mule.tooling.client.api.metadata.MetadataKeysRequest;
import org.mule.tooling.client.api.metadata.MetadataService;
import org.mule.tooling.client.internal.application.Application;
import org.mule.tooling.client.internal.serialization.XStreamFactory;

import com.thoughtworks.xstream.XStream;

import java.util.List;
import java.util.Optional;

/**
 * Public implementation of {@link MetadataService} that adapts Mule-API objects to DTOs
 *
 * @since 4.0
 */
public class ToolingMetadataServiceAdapter implements MetadataService, Command {

  private Application application;
  private MetadataProvider metadataProvider;
  private MetadataCache metadataCache;

  private XStream xStream;

  /**
   * Creates an instance of this service adapter.
   *
   * @param application {@link Application} to resolve the {@link ApplicationModel}. Non null.
   * @param metadataProvider {@link MetadataProvider} for resolving metadata using Mule Runtime. Non null.
   * @param metadataCache {@link MetadataCache} to fetch Metadata from cache. Non null.
   */
  public ToolingMetadataServiceAdapter(Application application, MetadataProvider metadataProvider, MetadataCache metadataCache) {
    checkNotNull(application, "application cannot be null");
    checkNotNull(metadataProvider, "metadataProvider cannot be null");
    checkNotNull(metadataCache, "metadataCache cannot be null");

    this.application = application;
    this.metadataProvider = metadataProvider;
    this.metadataCache = metadataCache;

    this.xStream = XStreamFactory.createXStream();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public org.mule.tooling.client.api.metadata.MetadataResult<org.mule.tooling.client.api.metadata.MetadataKeysContainer> getMetadataKeys(MetadataKeysRequest metadataKeysRequest)
      throws ServiceUnavailableException {

    ApplicationModel applicationModel = application.getApplicationModel();
    Optional<ComponentModel> componentModel = findComponentModel(applicationModel, metadataKeysRequest.getLocation());

    MetadataResult<MetadataKeysContainer> metadataKeys;
    if (!componentModel.isPresent()) {
      metadataKeys = failure(newFailure()
          .withMessage(format("No object found at location %s", metadataKeysRequest.getLocation().toString())).withFailureCode(
                                                                                                                               COMPONENT_NOT_FOUND)
          .onKeys());
    } else {
      String componentId = IdComponentModelSelector.getComponentId(componentModel.get());
      metadataKeys = (MetadataResult<MetadataKeysContainer>) xStream.fromXML((String) metadataCache
          .getMetadataKeys(new ImmutableMetadataCacheKeyInfo(componentId,
                                                             metadataKeysRequest
                                                                 .getLocation()
                                                                 .toString(),
                                                             null,
                                                             application
                                                                 .getProperties()),
                           () -> {
                             final MetadataResult<MetadataKeysContainer> runtimeMetadataResult =
                                 metadataProvider
                                     .getMetadataKeys(metadataKeysRequest);
                             List<MetadataFailure> failures = emptyList();
                             if (!runtimeMetadataResult.isSuccess()) {
                               failures = toMetadataFailuresDTO(runtimeMetadataResult.getFailures());
                             }
                             return new ImmutableMetadataResult(runtimeMetadataResult
                                 .isSuccess(), xStream
                                     .toXML(runtimeMetadataResult), failures);
                           }));
    }

    return toMetadataResultDTO(metadataKeys, () -> toMetadataKeysContainerDTO(metadataKeys.get()));
  }

  private Optional<ComponentModel> findComponentModel(ApplicationModel applicationModel, Location location) {
    final Reference<ComponentModel> foundComponentModelReference = new Reference<>();
    Optional<ComponentModel> globalComponent = applicationModel.findTopLevelNamedComponent(location.getGlobalName());
    globalComponent.ifPresent(componentModel -> findComponentWithLocation(componentModel, location)
        .ifPresent(foundComponentModel -> foundComponentModelReference.set(foundComponentModel)));
    return ofNullable(foundComponentModelReference.get());
  }

  private Optional<ComponentModel> findComponentWithLocation(ComponentModel componentModel, Location location) {
    if (componentModel.getComponentLocation().getLocation().equals(location.toString())) {
      return of(componentModel);
    }
    for (ComponentModel childComponent : componentModel.getInnerComponents()) {
      Optional<ComponentModel> foundComponent = findComponentWithLocation(childComponent, location);
      if (foundComponent.isPresent()) {
        return foundComponent;
      }
    }
    return empty();
  }

  @Override
  public Object invokeMethod(String methodName, String[] classes, String[] arguments) {
    switch (methodName) {
      case "getMetadataKeys": {
        checkState(arguments.length == 1,
                   format("Wrong number of arguments when invoking method created on %s", this.getClass().getName()));
        checkState(classes.length == 1 && classes[0].equals(MetadataKeysRequest.class.getName()),
                   format("Wrong type of arguments when invoking method created on %s", this.getClass().getName()));
        return serialize(getMetadataKeys(deserialize(arguments[0])));
      }
    }
    throw new IllegalArgumentException(format("Invalid arguments passed for calling method '%s' on '%s'", methodName,
                                              this.getClass()));
  }
}

==> mule-runtime-tooling-client-4.0.3-sources.jar: org/mule/tooling/client/internal/ToolingMetadataServiceAdapter.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */

package org.mule.tooling.client.internal;

import static com.google.common.base.Preconditions.checkNotNull;
import static java.lang.String.format;
import static java.util.Collections.emptyList;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.Optional.ofNullable;
import static org.mule.runtime.api.metadata.resolving.FailureCode.COMPONENT_NOT_FOUND;
import static org.mule.runtime.api.metadata.resolving.MetadataFailure.Builder.newFailure;
import static org.mule.runtime.api.metadata.resolving.MetadataResult.failure;
import static org.mule.runtime.api.util.Preconditions.checkState;
import static org.mule.tooling.client.internal.MetadataPartsFactory.toMetadataFailuresDTO;
import static org.mule.tooling.client.internal.MetadataPartsFactory.toMetadataKeysContainerDTO;
import static org.mule.tooling.client.internal.MetadataPartsFactory.toMetadataResultDTO;
import static org.mule.tooling.client.internal.serialization.XStreamServerSerializer.deserialize;
import static org.mule.tooling.client.internal.serialization.XStreamServerSerializer.serialize;
import org.mule.datasense.enrichment.model.IdComponentModelSelector;
import org.mule.runtime.api.metadata.MetadataKeysContainer;
import org.mule.runtime.api.metadata.resolving.MetadataResult;
import org.mule.runtime.api.util.Reference;
import org.mule.runtime.config.internal.model.ApplicationModel;
import org.mule.runtime.config.internal.model.ComponentModel;
import org.mule.tooling.client.api.component.location.Location;
import org.mule.tooling.client.api.datasense.ImmutableMetadataCacheKeyInfo;
import org.mule.tooling.client.api.datasense.ImmutableMetadataResult;
import org.mule.tooling.client.api.datasense.MetadataCache;
import org.mule.tooling.client.api.exception.ServiceUnavailableException;
import org.mule.tooling.client.api.metadata.MetadataFailure;
import org.mule.tooling.client.api.metadata.MetadataKeysRequest;
import org.mule.tooling.client.api.metadata.MetadataService;
import org.mule.tooling.client.internal.application.Application;
import org.mule.tooling.client.internal.serialization.XStreamFactory;

import com.thoughtworks.xstream.XStream;

import java.util.List;
import java.util.Optional;

/**
 * Public implementation of {@link MetadataService} that adapts Mule-API objects to DTOs
 *
 * @since 4.0
 */
public class ToolingMetadataServiceAdapter implements MetadataService, Command {

  private Application application;
  private MetadataProvider metadataProvider;
  private MetadataCache metadataCache;

  private XStream xStream;

  /**
   * Creates an instance of this service adapter.
   *
   * @param application {@link Application} to resolve the {@link ApplicationModel}. Non null.
   * @param metadataProvider {@link MetadataProvider} for resolving metadata using Mule Runtime. Non null.
   * @param metadataCache {@link MetadataCache} to fetch Metadata from cache. Non null.
   */
  public ToolingMetadataServiceAdapter(Application application, MetadataProvider metadataProvider, MetadataCache metadataCache) {
    checkNotNull(application, "application cannot be null");
    checkNotNull(metadataProvider, "metadataProvider cannot be null");
    checkNotNull(metadataCache, "metadataCache cannot be null");

    this.application = application;
    this.metadataProvider = metadataProvider;
    this.metadataCache = metadataCache;

    this.xStream = XStreamFactory.createXStream();
  }

  /**
   * {@inheritDoc}
   */
  @Override
  public org.mule.tooling.client.api.metadata.MetadataResult<org.mule.tooling.client.api.metadata.MetadataKeysContainer> getMetadataKeys(MetadataKeysRequest metadataKeysRequest)
      throws ServiceUnavailableException {

    ApplicationModel applicationModel = application.getApplicationModel();
    Optional<ComponentModel> componentModel = findComponentModel(applicationModel, metadataKeysRequest.getLocation());

    MetadataResult<MetadataKeysContainer> metadataKeys;
    if (!componentModel.isPresent()) {
      metadataKeys = failure(newFailure()
          .withMessage(format("No object found at location %s", metadataKeysRequest.getLocation().toString())).withFailureCode(
                                                                                                                               COMPONENT_NOT_FOUND)
          .onKeys());
    } else {
      String componentId = IdComponentModelSelector.getComponentId(componentModel.get());
      metadataKeys = (MetadataResult<MetadataKeysContainer>) xStream.fromXML((String) metadataCache
          .getMetadataKeys(new ImmutableMetadataCacheKeyInfo(componentId,
                                                             metadataKeysRequest
                                                                 .getLocation()
                                                                 .toString(),
                                                             null,
                                                             application
                                                                 .getProperties()),
                           () -> {
                             final MetadataResult<MetadataKeysContainer> runtimeMetadataResult =
                                 metadataProvider
                                     .getMetadataKeys(metadataKeysRequest);
                             List<MetadataFailure> failures = emptyList();
                             if (!runtimeMetadataResult.isSuccess()) {
                               failures = toMetadataFailuresDTO(runtimeMetadataResult.getFailures());
                             }
                             return new ImmutableMetadataResult(runtimeMetadataResult
                                 .isSuccess(), xStream
                                     .toXML(runtimeMetadataResult), failures);
                           }));
    }

    return toMetadataResultDTO(metadataKeys, () -> toMetadataKeysContainerDTO(metadataKeys.get()));
  }

  private Optional<ComponentModel> findComponentModel(ApplicationModel applicationModel, Location location) {
    final Reference<ComponentModel> foundComponentModelReference = new Reference<>();
    Optional<ComponentModel> globalComponent = applicationModel.findTopLevelNamedComponent(location.getGlobalName());
    globalComponent.ifPresent(componentModel -> findComponentWithLocation(componentModel, location)
        .ifPresent(foundComponentModel -> foundComponentModelReference.set(foundComponentModel)));
    return ofNullable(foundComponentModelReference.get());
  }

  private Optional<ComponentModel> findComponentWithLocation(ComponentModel componentModel, Location location) {
    if (componentModel.getComponentLocation().getLocation().equals(location.toString())) {
      return of(componentModel);
    }
    for (ComponentModel childComponent : componentModel.getInnerComponents()) {
      Optional<ComponentModel> foundComponent = findComponentWithLocation(childComponent, location);
      if (foundComponent.isPresent()) {
        return foundComponent;
      }
    }
    return empty();
  }

  @Override
  public Object invokeMethod(String methodName, String[] classes, String[] arguments) {
    switch (methodName) {
      case "getMetadataKeys": {
        checkState(arguments.length == 1,
                   format("Wrong number of arguments when invoking method created on %s", this.getClass().getName()));
        checkState(classes.length == 1 && classes[0].equals(MetadataKeysRequest.class.getName()),
                   format("Wrong type of arguments when invoking method created on %s", this.getClass().getName()));
        return serialize(getMetadataKeys(deserialize(arguments[0])));
      }
    }
    throw new IllegalArgumentException(format("Invalid arguments passed for calling method '%s' on '%s'", methodName,
                                              this.getClass()));
  }
}
